<?php

/**
 * WAAVE Credit Card Direct
 *
 * We load it later to ensure WC is loaded first since we're extending it.
 *
 * @class       WC_WAAVE_Credit_Card_Direct
 * @extends     WC_Payment_Gateway_CC
 * @version     2.0.0
 * @package     WooCommerce/Classes/Payment
 */
class WC_WAAVE_Credit_Card_Direct extends WC_Payment_Gateway_CC {

    const PROD_URL = 'https://getwaave.co';
    const SANDBOX_URL = 'https://staging.getwaave.co';

    const PROD_VGS_ENV = 'live';
    const PROD_VGS_VAULT_ID = 'tntf3oywshf';
    const SANDBOX_VGS_ENV = 'sandbox';
    const SANDBOX_VGS_VAULT_ID = 'tntflxlh3fw';

    protected $logger;

    protected $validation_code;

    public $order_button_text = "Place order";

    /**
     * Constructor for the gateway.
     */
    public function __construct() {
        $this->id                 = 'waave_guest_credit_card';
        $this->method_title       = __( 'WAAVE Complete', 'waave-complete' );
        $this->method_description = __( 'WAAVE guest credit card direct.', '' );
      
        // Common fields
        $this->title        = 'Credit and Debit Cards Payment - powered by WAAVE';
        $this->has_fields   = true;

        // Configuration fields
        $settings = get_option( 'woocommerce_waave_checkout_settings' );
        $this->access_key = $settings['access_key'];
        $this->private_key = $settings['private_key'];
        $this->venue_id = $settings['venue_id'];
        $this->enabled = $settings['enabled'];
        $this->callback_url = add_query_arg( 'wc-api', 'WC_WAAVE_Checkout', home_url( '/' ) );

        $this->waave_url = self::PROD_URL;
        $this->vgs_env = self::PROD_VGS_ENV;
        $this->vgs_vault_id = self::PROD_VGS_VAULT_ID;
        if ( 'yes' === $settings['testmode'] ) {
            $this->waave_url = self::SANDBOX_URL;
            $this->vgs_env = self::SANDBOX_VGS_ENV;
            $this->vgs_vault_id = self::SANDBOX_VGS_VAULT_ID;
        };

        // Actions
        add_action( 'woocommerce_thankyou_waave_guest_credit_card', array( $this, 'thankyou_page' ) );
        add_action( 'wp_enqueue_scripts', array($this, 'plugin_assets') );
        add_action( 'admin_enqueue_scripts', array($this, 'admin_assets') );
        add_action( 'wp_enqueue_scripts', array($this, 'plugin_vgs_assets') );
    }

    public function plugin_vgs_assets() {
        wp_enqueue_script( 'woocommerce_waave_vgs_plugin', 'https://js.verygoodvault.com/vgs-collect/2.18.1/vgs-collect.js', array(),  time() );
        wp_enqueue_script( 'woocommerce_waave_vgs_add_card', plugins_url( 'assets/js/vgs_add_card.js', WC_WAAVE_MAIN_FILE ), array(), '5.4.1.'.rand(1, 10000) );
        wp_localize_script( 
			'woocommerce_waave_vgs_plugin', 
			'vgs_config', 
			array(
				'env'                   => $this->vgs_env,
				'vault_id'              => $this->vgs_vault_id,
				'waave_form_id'         => $this->id,
			) 
            
		);
    }

    public function plugin_assets() {
        wp_enqueue_style( 'woocommerce_waave_style', plugins_url( 'assets/css/style.css', WC_WAAVE_MAIN_FILE ), array(), time() );
        wp_enqueue_script( 'woocommerce_waave', plugins_url( 'assets/js/index.js', WC_WAAVE_MAIN_FILE ), array(), time(), true );
    }

    public function admin_assets() {
        wp_enqueue_script( 'woocommerce_admin_waave', plugins_url( 'assets/js/admin.js', WC_WAAVE_MAIN_FILE ), array(), time(), true );
    }

    private function log( $message ) {
        if ( empty( $this->logger ) ) {
            $this->logger = new WC_Logger();
        }

        $this->logger->add( 'waave', $message );
    }

    private function get_posted_value( $key, $default = '', $is_detail = false) {
        $value = $default;
        $name  = $this->id . '-' . $key;
        if ($is_detail) {
            $name = $key;
        }

        if ( isset( $_POST[ $name ] ) ) {
            $value = is_string( $_POST[ $name ] ) ? trim( $_POST[ $name ] ) : $_POST[ $name ];
        }

        return $value;
    }

    private function get_calling_codes() {
        $codes = wp_cache_get( 'calling-codes', 'countries' );

        if ( ! $codes ) {
            $codes = include WC()->plugin_path() . '/i18n/phone.php';
            wp_cache_set( 'calling-codes', $codes, 'countries' );
        }

        return $codes;
    }


    public function get_icon() {
        $icons_str = '<img src="' . WC_WAAVE_PLUGIN_URL . '/assets/images/logo.png" class="" width="100" alt="WAAVE" />';

        return apply_filters( 'woocommerce_gateway_icon', $icons_str, $this->id );
    }

    public function payment_fields() {
        wp_enqueue_script( 'wc-credit-card-form' );

        $top = array("+1", "+33", "+44", "+61", "+64");
        $calling_codes = array();
        foreach ($this->get_calling_codes() as $value) {
            if (is_array($value) || !$value || in_array($value, $top)) {
                continue;
            }
            $calling_codes[] = $value;
        }
        $calling_codes = array_unique($calling_codes);
        sort($calling_codes);
        $codes = array_merge($top, $calling_codes);

        $options = '';
        foreach ($codes as $value) {
            $selected = "";
            if (1 == $value) {
                $selected = "selected";
            }

            $options .= "<option value='{$value}' $selected>$value</option>";
        }
        ?>

        <fieldset id="wc-<?php echo esc_attr( $this->id ); ?>-cc-form" class='wc-credit-card-form wc-payment-form'>
            <?php do_action( 'woocommerce_credit_card_form_start', $this->id ); ?>
                <form id="frmAddCard">
                    <div class="card-body py-3">
                        <p class="small alert alert-danger" style="display: none"></p>
                        <p class="form-row form-row-wide form-row-wide">
                            <strong>Enter card information</strong>
                        </p>
                        <div class="form-group">
                            <div class="input-group">
                                <label for="">Card number&nbsp;<span class="required">*</span></label>
                                <div id="cc-number" class="form-field"></div>
                            </div>
                        </div>
                        <div class="form-group form-row-first">
                            <div class="input-group">
                                <label for="">Expiration (MM/YY)&nbsp;<span class="required">*</span></label>
                                <div id="cc-expiration-date" class="form-field"></div>
                            </div>
                        </div>
                        <div class="form-group form-row-last">
                            <div class="input-group">
                                <label for="">Card Security Code&nbsp;<span class="required">*</span></label>
                                <div id="cc-cvc" class="form-field"></div>
                            </div>
                        </div>
                    </div>
                </form>
                <?php do_action( 'woocommerce_credit_card_form_end', $this->id ); ?>
            <input type="hidden" id="waaveUrl" value="<?php echo $this->waave_url . '/complete'; ?>" />
            <div class="clear"></div>
        </fieldset>

        <?php
    }

    public function validate_fields() {
        $products = array();
        $cart_contents = WC()->cart->get_cart();
        foreach ($cart_contents as $value) {
            $product = $value['data'];
            if (empty($product)) {
                continue;
            }

            $product_id = $product->get_id();
            $categories = $product->get_category_ids();
            if ('variation' == $product->get_type()) {
                $product_id = $product->get_parent_id();
                $parent = wc_get_product($product_id);
                $categories = $parent->get_category_ids();
            }

            $products[] = array(
                'id' => $product_id,
                'name' => $product->get_name(),
                'sku' => $product->get_sku(),
                'price' => $product->get_price(),
                'quantity' => $value['quantity'],
                'categories' => $categories
            );
        }

        $shipping_first_name = $this->get_posted_value('billing_first_name', '', true);
        $shipping_last_name = $this->get_posted_value('billing_last_name', '', true);
        $billing_country = $this->get_posted_value('billing_country', '', true);
        $billing_city = $this->get_posted_value('billing_city', '', true);
        $billing_state = $this->get_posted_value('billing_state', '', true);
        if (empty($billing_state)) {
            $billing_state = "MUnicipality";
        }
        $billing_address_1 = $this->get_posted_value('billing_address_1', '', true);
        $billing_address_2 = $this->get_posted_value('billing_address_2', '', true);
        $billing_postcode = $this->get_posted_value('billing_postcode', '', true);

        $billing = array(
            'billing_first_name' => $shipping_first_name,
            'billing_last_name' => $shipping_last_name,
            'billing_country' => $billing_country,
            'billing_address_1' => $billing_address_1,
            'billing_address_2' => $billing_address_2,
            'billing_city' => $billing_city,
            'billing_state' => $billing_state,
            'billing_postcode' => $billing_postcode
        );

        $ship_to_different_address = $this->get_posted_value('ship_to_different_address', 0, true);
        if ($ship_to_different_address) {
            $shipping_country = $this->get_posted_value('shipping_country', '', true);
            $shipping_city = $this->get_posted_value('shipping_city', '', true);
            $shipping_state = $this->get_posted_value('shipping_state', '', true);
            if (empty($shipping_state)) {
                $shipping_state = "MUnicipality";
            }
            $shipping_address_1 = $this->get_posted_value('shipping_address_1', '', true);
            $shipping_address_2 = $this->get_posted_value('shipping_address_2', '', true);
            $shipping_postcode = $this->get_posted_value('shipping_postcode', '', true);
        } else {
            $shipping_country = $billing_country;
            $shipping_city = $billing_city;
            $shipping_state = $billing_state;
            $shipping_address_1 = $billing_address_1;
            $shipping_address_2 = $billing_address_2;
            $shipping_postcode = $billing_postcode;
        }

        $shipping = array(
            'shipping_country' => $shipping_country,
            'shipping_address_1' => $shipping_address_1,
            'shipping_address_2' => $shipping_address_2,
            'shipping_city' => $shipping_city,
            'shipping_state' => $shipping_state,
            'shipping_postcode' => $shipping_postcode
        );

        $venue_id = get_option('waave_compliance_venue_id');
        if (is_plugin_inactive('waave-compliance/waave-compliance.php')) {
            $venue_id = $this->venue_id;
        }

        $body = array(
            'venue_id' => $venue_id,
            'amount'   => WC()->cart->get_cart_contents_total(),
            'email'    => $this->get_posted_value('billing_email', '', true),
            'products' => $products,
            'shipping' => $shipping,
            'billing'  => $billing
        );

        $url = $this->waave_url . '/compliance/validate';
        $options = array(
            'body' => $body
        );

        $request  = wp_remote_post( $url, $options );
        $response = json_decode( wp_remote_retrieve_body( $request ), true );

        if (isset($response['message']) && empty($response['success'])) {
            wc_add_notice( '<strong>' . $response['message'] . '</strong>', 'error' );
        }

        // Save validation code
        if (isset($response['validation_code'])) {
            $this->validation_code = $response['validation_code'];
        }

        return true;
    }

    /**
     * Process the payment and return the result
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment( $order_id ) {
        $isAddCardError = $this->get_posted_value( 'is_error_add_card', '', true);
        if ($isAddCardError) {
            $errorMsg = $this->get_posted_value( 'error_add_card_form', '', true);
            if ($errorMsg) wc_add_notice( '<strong>' . $errorMsg . '</strong>', 'error' );
            else wc_add_notice( '<strong>Something went wrong. Refresh to try again.</strong>', 'error' );
            return false;
        }
        
        $order = wc_get_order( $order_id );

        $data_to_send = array(
            // WAAVE details
            'access_key'   => $this->access_key,
            'venue_id'     => $this->venue_id,
            'return_url'   => $this->get_return_url( $order ),
            'callback_url' => $this->callback_url,
            'cancel_url'   => $order->get_cancel_order_url(),
            'validation_code' => $this->validation_code,

            // Order details
            'amount'       => $order->get_total(),
            'reference_id' => (string) $order->get_id(),
            'currency'     => $order->get_currency(),
            'customer_ip'  => $order->get_customer_ip_address()
        );

        $card_token = $this->get_posted_value('card-token', '');

        $card = array(
            'card_token'    =>  $card_token
        );

        $orderData = $order->get_data();
        $billing = isset($orderData['billing']) ? $orderData['billing'] : array();
        if ( $billing ) {
            $billing['phone_code'] = WC()->countries->get_country_calling_code( $billing['country'] );
        }
        if (empty($billing['state'])) {
            $billing['state'] = "MUnicipality";
        }

        $shipping = $billing;
        $ship_to_different_address = $this->get_posted_value('ship_to_different_address', 0, true);
        if ($ship_to_different_address && isset($orderData['shipping'])) {
            $shipping = $orderData['shipping'];
            if ( $shipping ) {
                $shipping['phone_code'] = WC()->countries->get_country_calling_code( $shipping['country'] );
            }
        }
        if (empty($shipping['state'])) {
            $shipping['state'] = "MUnicipality";
        }
        $data_to_send['card_information'] = $card;
        $data_to_send['billing'] = $billing;
        $data_to_send['shipping'] = $shipping;
        // Send data
        $url = $this->waave_url . '/complete/transactions';

        $secretKey  = $this->private_key;
        $body       = json_encode($data_to_send);
        $signature  = hash( "sha256", $secretKey . $url . $body );

        $options = array(
            'body' => $data_to_send,
            'timeout' => 300,
            'headers' => [
                'X-Api-Signature' => $signature
            ]
        );

        $request  = wp_remote_post( $url, $options );
        $response = json_decode( wp_remote_retrieve_body( $request ), true );

        if (isset($response['message']) && empty($response['success'])) {
            throw new Exception($response['message']);
        }

        if ( isset( WC()->cart ) ) {
            WC()->cart->empty_cart();
        }

        // Return thankyou redirect
        return array(
            'result'    => 'success',
            'redirect'  => $this->get_return_url($order)
        );
    }

    /**
     * Thank you page.
     * 
     * @param int $order_id
     *
     */
    public function thankyou_page( $order_id ) {
        echo '<p>' . __( 'Thank you for your order with WAAVE.', 'waave-complete' ) . '</p>';
    }
}