<?php

/**
 * Plugin Name: WooCommerce WAAVE Complete
 * Description: WAAVE Checkout and WAAVE guest credit card payment.
 * Author: WAAVE
 * Author URI: https://getwaave.com/
 * Version: 5.4.2
 * Requires at least: 4.6
 * WC requires at least: 4.0
 * WC tested up to: 5.0
 * Text Domain: waave-complete
 * Domain Path: /i18n/languages/
 *
 */

defined( 'ABSPATH' ) or exit;

define( 'WC_WAAVE_PLUGIN_URL', untrailingslashit( plugins_url( basename( plugin_dir_path( __FILE__ ) ), basename( __FILE__ ) ) ) );
define( 'WC_WAAVE_PLUGIN_PATH', untrailingslashit( plugin_dir_path( __FILE__ ) ) );
define( 'WC_WAAVE_MAIN_FILE', __FILE__ );


// Make sure WooCommerce is active
if ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
    return;
}

/**
 * Initialize the gateway.
 * @since 1.0.0
 */
function wc_waave_complete_init() {
    if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
        return;
    }

    require_once ABSPATH . 'wp-admin/includes/plugin.php';
    require_once( plugin_basename( 'includes/class-wc-waave-checkout.php' ) );
    require_once( plugin_basename( 'includes/class-wc-waave-credit-card-direct.php' ) );
}
add_action( 'plugins_loaded', 'wc_waave_complete_init', 11 );


function wc_waave_complete_load_textdomain() {
    load_plugin_textdomain( 'waave-complete', false, dirname( plugin_basename(__FILE__) ) . '/i18n/languages/' );
}
add_action('plugins_loaded', 'wc_waave_complete_load_textdomain');

/**
 * Add the gateway to WC Available Gateways
 * 
 * @since 1.0.0
 * @param array $gateways all available WC gateways
 * @return array $gateways all WC gateways + waave complete
 */
function wc_add_waave_complete_to_gateways( $gateways ) {
    $gateways[] = 'WC_WAAVE_Checkout';
    $gateways[] = 'WC_WAAVE_Credit_Card_Direct';
    return $gateways;
}
add_filter( 'woocommerce_payment_gateways', 'wc_add_waave_complete_to_gateways' );


/**
 * Adds plugin page links
 * 
 * @since 1.0.0
 * @param array $links all plugin links
 * @return array $links all plugin links + our custom links (i.e., "Settings")
 */
function wc_waave_complete_plugin_links( $links ) {

    $plugin_links = array(
        '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout&section=waave_checkout' ) . '">' . __( 'Configure', 'wc-gateway-waave' ) . '</a>'
    );

    return array_merge( $plugin_links, $links );
}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'wc_waave_complete_plugin_links' );

add_filter( 'woocommerce_checkout_fields', 'bbloomer_set_checkout_field_input_value_default' );
 
function bbloomer_set_checkout_field_input_value_default($fields) {
    $fields['billing']['billing_country']['default'] = 'Uruguay';
    return $fields;
}